# frozen_string_literal: true

require 'yaml'

module FIS
  # Handles the global configuration of the gem via ~/.flatiron-school/configuration.yml
  class Configuration
    attr_reader :file_path, :file_name, :file_full_path, :config

    def initialize
      @file_path = File.join(Dir.home, '.flatiron-school')
      @file_name = 'configuration.yml'
      @file_full_path = File.join(@file_path, @file_name)

      @config = read
    end

    def fetch(*keys)
      key = keys.pop
      keys.inject(@config, :fetch)[key]
    end

    def set(*keys)
      key = keys.pop
      keys.inject(@config, :fetch)[key] = yield

      write!
    end

    def unset(*keys)
      set(*keys) do
        key = keys.pop
        keys.inject(default, :fetch)[key]
      end
    end

    def read
      return default unless File.exist?(@file_full_path)

      deep_merge(
        default,
        YAML.safe_load(
          File.open(@file_full_path).read,
          [Symbol]
        )
      )
    end

    def reread
      @config = read
    end

    def write!
      Dir.mkdir(@file_path, 0o700) unless Dir.exist?(@file_path)

      File.open(@file_full_path, 'w') do |file|
        yaml = deep_merge(default, @config).to_yaml
        file.write(yaml)
      end
    end

    private

    def default
      {
        identity: {
          portal: {
            token: nil
          },
          github: {
            id: nil,
            username: nil
          }
        }
      }
    end

    def deep_merge(this_hash, other_hash)
      this_hash.merge(other_hash) do |_key, this_val, other_val|
        if this_val.is_a?(::Hash) && other_val.is_a?(::Hash)
          deep_merge(this_val, other_val)
        else
          other_val
        end
      end
    end
  end
end
